"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = exports.fetchHistoric90d = exports.fetchHistoric = exports.fetch = void 0;
const fast_xml_parser_1 = require("fast-xml-parser");
// http://www.ecb.europa.eu/stats/policy_and_exchange_rates/euro_reference_exchange_rates/html/index.en.html
async function fetch() {
    const result = await get('http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml');
    const rates = parse(result);
    if (rates.length !== 1) {
        throw new Error(`Expected result to contain one single entry, but got ${rates.length}`);
    }
    return rates[0];
}
exports.fetch = fetch;
async function fetchHistoric() {
    return parse(await get('https://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist.xml'));
}
exports.fetchHistoric = fetchHistoric;
async function fetchHistoric90d() {
    return parse(await get('https://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist-90d.xml'));
}
exports.fetchHistoric90d = fetchHistoric90d;
async function get(url) {
    const result = await global.fetch(url);
    return await result.text();
}
function parse(string) {
    const data = new fast_xml_parser_1.XMLParser({ ignoreAttributes: false, isArray: () => true }).parse(string);
    const result = [];
    const entries = data['gesmes:Envelope'][0]['Cube'][0]['Cube'];
    if (typeof entries !== 'object') {
        throw new Error('Result data does not have the expected structure');
    }
    for (const current of entries) {
        const time = current?.['@_time']?.[0];
        assertString(time, 'time');
        const rates = {};
        for (const item of current['Cube']) {
            const currency = item['@_currency']?.[0];
            assertString(currency, 'curency');
            const rateString = item['@_rate']?.[0];
            assertString(rateString, 'rate');
            const rate = parseFloat(rateString);
            rates[currency] = rate;
        }
        result.push({ time, rates });
    }
    return result;
}
exports.parse = parse;
function assertString(value, valueName) {
    if (typeof value !== 'string') {
        throw new Error(`Expected ${valueName} to be a string`);
    }
}
// CLI only when module is not require'd
if (require.main === module) {
    (async () => {
        const result = await fetch();
        console.log(JSON.stringify(result, null, 2));
    })().catch(() => process.exit(1));
}
